/**
 * SALSA/World Wide Computer Project
 *
 * Location Server - Universal Actor Name Daemon
 * Provides the location (Universal Actor Location) of an actor running on
 * the World Wide Computer from its name (Universal Actor Name).
 *
 * By Gregory Haik and Carlos Varela.  v0.1  June, 1999
 *
 */
package wwc.naming.locationServer;

import java.net.*;
import java.io.*;
import java.util.*;
import wwc.naming.*;

/**
 * Contains the main method of the UAN daemon.
 * Adapted from UANDaemon by Robin D. Toll
 *
 * @author Robin D. Toll
 * @version 0.12.3.2001
 */
public class UANConsoleDaemon {
	static int portNumber = 3030;							// The default port number is set to 3030.
	static ServerSocket serverSocket = null;				// The server will listen via this socket.
	static String protocolVersion = UANProtocol.VERSION;	// This server implements the very first version of UAN protocol.
	static String serverVersion = new String("uand/0.1");	// Location server version.
	static Hashtable locationMap;							// Mapping from UAN's to UAL's.
	private String response;								// Used for communications.
	private BufferedReader in;								// Used for communications.
	private PrintWriter out;								// Used for communications.
	private boolean flag = true;							// Flag variable for while loop.

	public static void main(String[] argv) {
		new UANConsoleDaemon(argv);
	}

	public UANConsoleDaemon(String[] argv) {
		for (int x=0; x<argv.length; x++) {
			if (argv[x].equals("-h"))
				helpMessage();
			else if (argv[x].equals("-v"))
				printServerVersion();
			else if (argv[x].equals("-p")) {
				x++;
				try {
					portNumber = Integer.parseInt(argv[x]);
				} catch (Exception e) {
					helpMessage("Invalid port number: "+argv[x]);
				}
			}
			else
				helpMessage("illegal option");
		}
		UANDConsole t = new UANDConsole(this);
		t.setDaemon(true);
		t.start();
		try {
			serverSocket = new ServerSocket(portNumber);
		} catch (IOException e) {
			System.out.println("Could not listen on port: "+portNumber+", "+e);
			System.exit(1);
		}
		locationMap = new Hashtable();
		while (flag) {
			Socket clientSocket = null;
			try {
				clientSocket = serverSocket.accept();
				in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
				out = new PrintWriter(clientSocket.getOutputStream(), true);
			} catch (IOException e) {
				if (flag == false) {
					System.out.println("Shutting Down Name Server...");
					System.exit(0);
				} else {
					System.out.println("Accept failed: "+portNumber+", "+e);
					System.exit(1);
				}
			}
			try {
				try {
					LocationRequest request = new LocationRequest(in, locationMap);  
					response = request.process();
				} catch (BadUANPRequestException e) {
					response = new String(UANProtocol.VERSION
						+" "
						+UANProtocol.BAD_REQ_STATUS_CODE
						+" "
						+UANProtocol.BAD_REQ_STATUS_STR
						+"\n");
				}
				System.out.println("Response : "+response);
				out.println(response);
				in.close();
				out.close();
				clientSocket.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	static void printServerVersion() {
		System.out.println("Universal Actor Name Daemon.");
		System.out.println(serverVersion);
		System.exit(0);
	}

	static void helpMessage(String s) {
		System.out.println(s);
		UANConsoleDaemon.printHelpMessage();
		System.exit(1);
	}

	static void helpMessage() {
		UANConsoleDaemon.printHelpMessage();
		System.exit(0);
	}

	static void printHelpMessage() {
		System.out.println("usage:");
		System.out.println("  java ...UANConsoleDaemon");
		System.out.println("  java ...UANConsoleDaemon -h");
		System.out.println("  java ...UANConsoleDaemon -v");
		System.out.println("  java ...UANConsoleDaemon -p portNumber");
		System.out.println("options:");
		System.out.println("  -h : print this message");
		System.out.println("  -v : print version");
		System.out.println("  -p portNumber : set the listening port to portNumber");
		System.out.println("                  default port number is 3030");
	}

	void kill() {
		flag = false;
		try {
			serverSocket.close();
		} catch (Exception e) {
			serverSocket = null;
		}
	}
}

class UANDConsole extends Thread {
	UANConsoleDaemon myDaemon;
	private boolean flag = true;

	public UANDConsole(UANConsoleDaemon u) {
		myDaemon = u;
	}

	public void run() {
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		String in = null;
		while (flag) {
			try {
				in = br.readLine();
			} catch (Exception e) {
				flag = false;
				break;
			}
			if (in == null)
				in = "";
			if (in.length() == 0) {
				// Do nothing; ignore empty input lines.
			} else if (in.toUpperCase().equals(new String("QUIT").substring(0, in.length())))
				flag = false;
			else if (in.toUpperCase().equals(new String("LIST").substring(0, in.length())))
				listElements();
			else if (in.equals("?"))
				helpMessage();
			else
				System.out.println("Unknown Command: "+in);
		}
		myDaemon.kill();
	}

	private void helpMessage() {
		System.out.println();
		System.out.println("Q(UIT): Shut down the server and exit.");
		System.out.println("?: This message.");
	}

	private void listElements() {
		Enumeration e1 = myDaemon.locationMap.keys();
		Enumeration e2 = myDaemon.locationMap.elements();
		System.out.println();
		System.out.println("Entries:");
		System.out.println("-------");
		int cnt = 0;
		while ((e1.hasMoreElements()) && (e2.hasMoreElements())) {
			System.out.println(e1.nextElement()+" --> "+e2.nextElement());
			cnt++;
		}
		if (cnt == 0)
			System.out.println("<NONE");
		else {
			System.out.println("-------");
			System.out.println(cnt+" total entries");
		}
	}
}
