/* SALSA/World Wide Computer Porject
 *
 * Several utility classes are packaged in wwc.util.
 * This classes are used by different packages.
 *
 * By Gregory Haik and Carlos Varela.  v0.1  June, 1999
 *
 */

package wwc.util;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.lang.reflect.Method;
import java.lang.Thread;
import java.net.ServerSocket;
import java.net.Socket;


/**
 * This class implements a input gate for any kind of serializable object.
 * When started via the start() method, it listens
 * method start(), the gate opens a server socket that listens to a port
 * specified by 'a'. As soon as the gate receives an object, it calls a
 * callback defined in the A class. The name of the callback method must
 * be passed as argument of the gate's constructor. Notice that the compiler
 * does not check the callback signature before the execution. 
 */
public class ObjectInputGate extends Thread  {
    protected int portNumber;
    protected Method callback;
    protected Object target;

    protected ServerSocket serverSocket;

/**
 * Creates new gate, that is, starts listening for incoming objects.
 *
 * @param pn the port number the gate will listen to.
 * @param callbackName the name of the method that will be activated when an object is received.
 * @param t the object that will receive the callback.
 * Note : any uncaught exception thrown in the callback will be forgotten !
 */
    public ObjectInputGate(int pn, String callbackName, Object t) 
	throws IOException, CallbackNotImplementedException {
	    portNumber = pn;
	    target = t;
	    Class[] args = {(new Object()).getClass()};
	    try {
		callback = t.getClass().getMethod(callbackName, args);
	    } catch (Exception e) {
		throw new CallbackNotImplementedException("Method "
							  + callbackName
							  + "(Object) not public or not found in class "
							  + t.getClass().getName()
							  + ".");
	    }	    
	try {
	    serverSocket = new ServerSocket(pn);
	} catch (Exception e) {
	    throw new IOException("Could not listen on port: " + 
				  portNumber + ", " + e);
	}
	this.start();
    }

/**
 * overrides the Thread.run() method. It stands for the behavior of the gate.
 */
    public void run()
    {
    	ObjectInputStream iStream;
	    Object receivedObject = null;

	    // This loop only broken by a break
 	    while (true) {
            // Waits for a connection, and then sets up the 
    	    Socket clientSocket = null;
	        try {
    	    	clientSocket = serverSocket.accept();
	    	    iStream = new ObjectInputStream(
		    	      new BufferedInputStream (clientSocket.getInputStream()));
// 		        iStream.enableResolveObject(true);
    		    receivedObject = iStream.readObject();
	    	    iStream.close();
		        clientSocket.close();
    	    } catch (Exception e) {
    	        // There was a problem with resolving the connection...
	    	    e.printStackTrace();
		        continue;
    	    }

            // If we really did get an object...
	        if (receivedObject != null)
	        {
	            // Tell our "parent" object.
		        Object[] args = {receivedObject};
        		try {
	        	    callback.invoke(target, args);
		        } catch (java.lang.reflect.InvocationTargetException ite) {
		            // If that doesn't work...
		            Throwable targetException = ite.getTargetException();
    		        if (targetException instanceof ThreadDeath)
 	    	    	    throw new ThreadDeath();
    		        else
	    	        {
                        System.err.println(targetException+
	    	    			   " processing message:"+
		    	    		   callback);
    			        targetException.printStackTrace();
        		    }
	        	    continue;
		        } catch (IllegalAccessException iae) {
		            System.err.println(iae+" processing message:"+callback);
        		} catch (Exception e) {
	        	    e.printStackTrace();
		            continue;
    		    }
       	    }
        }
    }
}
